package bnrobot.classification.knn;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.TreeSet;

import bnrobot.core.ClassificationResult;
import bnrobot.core.Classificator;
import bnrobot.core.DocCollection;
import bnrobot.core.Similarity;
import bnrobot.core.TfIdfElement;
import bnrobot.core.models.DocumentElement;
import bnrobot.core.models.LabeledDocument;
import bnrobot.core.models.Subject;

public class NearestNeighbor implements Classificator {
	private final int k = 13;

	private Collection<TfIdfElement<LabeledDocument>> labeledDocuments = null;

	private Collection<TfIdfElement<LabeledDocument>> getElements(
			DocCollection collection) {
		if (this.labeledDocuments == null
				|| this.labeledDocuments.size() != collection
						.getLabeledDocumentsSize()) {
			this.labeledDocuments = new ArrayList<TfIdfElement<LabeledDocument>>();
			for (LabeledDocument document : collection.getLabeledDocuments()) {
				this.labeledDocuments.add(new TfIdfElement<LabeledDocument>(
						collection.getIdf(), document));
			}
		}
		return this.labeledDocuments;
	}

	public ClassificationResult categoryOf(Subject subject,
			DocCollection collection) {

		TfIdfElement<Subject> subjectIdf = new TfIdfElement<Subject>(
				collection.getIdf(), subject);

		Collection<TfIdfElement<LabeledDocument>> documentos = this
				.getElements(collection);

		if (documentos.size() == 0) {
			return new ClassificationResult(null, Double.valueOf(0));
		}

		TreeSet<Similarity<LabeledDocument, Subject>> similarities = new TreeSet<Similarity<LabeledDocument, Subject>>();
		for (TfIdfElement<LabeledDocument> document : documentos) {
			similarities.add(new Similarity<LabeledDocument, Subject>(document,
					subjectIdf));
		}

		int i = 0;
		TreeMap<Integer, Double> map = new TreeMap<Integer, Double>();
		Similarity<LabeledDocument, Subject> maiorSimilaridade = similarities
				.last();
		for (Similarity<LabeledDocument, Subject> similarity : similarities
				.descendingSet()) {

			if (i++ == this.k) {
				break;
			}
			Integer label = similarity.getElementA().getLabel();
			Double sum = map.containsKey(label) ? map.get(label)
					+ similarity.getValue() : similarity.getValue();
			map.put(label, sum);
		}

		Iterator<Entry<Integer, Double>> iterator = map.entrySet().iterator();
		Entry<Integer, Double> max = iterator.next();
		while (iterator.hasNext()) {
			Entry<Integer, Double> ele = iterator.next();
			if (ele.getValue() > max.getValue()) {
				max = ele;
			}
		}
		Integer mm = max.getKey();

		return new ClassificationResult(mm, Double.valueOf(maiorSimilaridade
				.getValue()));
	}

	public void learn(DocumentElement element, Integer label) {
		// this.documents.add(new LabeledDocument(element, label));
	}

	public void commit() {
		// Não precisa commitar ;)

	}
}
