package bnrobot.clustering.hierarchical;

import java.util.ArrayList;
import java.util.List;

import bnrobot.core.ArrayOperations;
import bnrobot.core.models.DocumentElement;
import bnrobot.vectorSpace.VSElement;

public class Group implements VSElement {

	private int id;
	private int[] indexes;
	private double[] tf;
	private final List<DocumentElement> elements;

	public Group(int id, DocumentElement element) {
		this.id = id;
		this.elements = new ArrayList<DocumentElement>();
		this.elements.add(element);

		this.tf = element.getValues();
		this.indexes = element.getIndexes();
	}

	public Group(int id, Group a, Group b) {
		this.id = id;
		this.elements = new ArrayList<DocumentElement>();
		this.elements.addAll(a.elements);
		this.elements.addAll(b.elements);

		this.computeTf(a, b);
	}

	private void computeTf(Group a, Group b) {
		int size = ArrayOperations.unionsSize(a.indexes, b.indexes);
		this.indexes = new int[size];
		this.tf = new double[size];

		int pa = 0, pb = 0, pi = 0;

		while (pa < a.indexes.length || pb < b.indexes.length) {
			if ((pa >= a.indexes.length)
					|| (pb < b.indexes.length && a.indexes[pa] > b.indexes[pb])) {
				this.indexes[pi] = b.indexes[pb];
				this.tf[pi] = b.tf[pb];
				pi++;
				pb++;
			} else if ((pb >= b.indexes.length)
					|| (pa < a.indexes.length && b.indexes[pb] > a.indexes[pa])) {
				this.indexes[pi] = a.indexes[pa];
				this.tf[pi] = a.tf[pa];
				pi++;
				pa++;
			} else {
				this.indexes[pi] = a.indexes[pa];
				this.tf[pi] = Math.log1p(Math.exp(a.tf[pa])
						+ Math.exp(b.tf[pb]) - 1);
				pi++;
				pa++;
				pb++;
			}
		}

	}

	public int[] getIndexes() {
		return this.indexes;
	}

	public double[] getValues() {
		return this.tf;
	}

	public void setId(int id) {
		this.id = id;
	}

	public int getId() {
		return this.id;
	}

	public List<DocumentElement> getElements() {
		return this.elements;
	}

}
