package bnrobot.core;

import bnrobot.vectorSpace.Operations;
import bnrobot.vectorSpace.VSElement;

/**
 * 
 * @author Willian Gigliotti
 * 
 * @param <A>
 * @param <B>
 */
public class Similarity<A extends VSElement, B extends VSElement> implements
		Comparable<Similarity<?, ?>> {

	private double similarity = -1;
	private TfIdfElement<A> elementA;
	private TfIdfElement<B> elementB;

	public Similarity(TfIdfElement<A> elementA, TfIdfElement<B> elementB) {
		this.elementA = elementA;
		this.elementB = elementB;
	}

	public Similarity(A elementA, B elementB, DocCollection collection) {
		this(new TfIdfElement<A>(collection.getIdf(), elementA),
				new TfIdfElement<B>(collection.getIdf(), elementB));
	}

	public Similarity(A elementA, TfIdfElement<B> elementB,
			DocCollection collection) {
		this(new TfIdfElement<A>(collection.getIdf(), elementA), elementB);
	}

	public Similarity(TfIdfElement<A> elementA, B elementB,
			DocCollection collection) {
		this(elementA, new TfIdfElement<B>(collection.getIdf(), elementB));
	}

	/**
	 * 
	 * @return elemento a esquerda
	 */
	public A getElementA() {
		return this.elementA.getElement();
	}

	/**
	 * 
	 * @return elemento a direita
	 */
	public B getElementB() {
		return this.elementB.getElement();
	}

	/**
	 * 
	 * @return o valor da similaridade entre os elementos A e B
	 */
	public double getValue() {
		if (this.similarity < 0)
			this.similarity = Operations.dotProduct(this.elementA, this.elementB);
		return this.similarity;
	}

	/**
	 * {@inheritDoc}
	 */
	public int compareTo(Similarity<?, ?> o) {
		return Double.compare(this.getValue(), o.getValue());
	}

}
