package bnrobot.core;

import bnrobot.vectorSpace.VSElement;

public class TfIdfElement<E extends VSElement> implements VSElement {

	private E element;
	private double[] idf;
	private int[] indexes;
	private double[] tfidf;

	public TfIdfElement(double[] idf, E element) {
		this.idf = idf;
		this.element = element;
	}

	/**
	 * Calcula o valor tf-idf
	 */
	private void computeTfIdf() {
		this.indexes = this.element.getIndexes();
		double[] tf = this.element.getValues();

		this.tfidf = new double[indexes.length];
		double size = 0;
		

		// calculando o vetor de frequencia
		for (int i = 0; i < indexes.length; i++) {

			double val = tf[i] * this.idf[(int) indexes[i]];

			tfidf[i] = val;
			size += val * val;
		}
		size = Math.sqrt(size);

		// dividindo o vetor pela norma
		for (int i = 0; i < indexes.length; i++) {
			tfidf[i] = tfidf[i] / size;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public double[] getValues() {
		if (this.indexes != this.element.getIndexes()) {
			this.computeTfIdf();
		}
		return this.tfidf;
	}

	/**
	 * {@inheritDoc}
	 */
	public int[] getIndexes() {
		if (this.indexes != this.element.getIndexes()) {
			this.computeTfIdf();
		}
		return this.indexes;
	}

	/**
	 * @return o elemento
	 */
	public E getElement() {
		return this.element;
	}

}
