package bnrobot.core;

import java.io.Serializable;

import bnrobot.vectorSpace.VSElement;

/**
 * Classe com implementação básica para um {@link VSElement} para a
 * representação de espaço vetorial de documentos.<br>
 * 
 * @author Willian Gigliotti
 * 
 */
public class VSElementImpl implements VSElement, Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected int[] indexes;
	protected double[] tf;

	/**
	 * Seta os valores do vetor
	 * 
	 * @param indexes
	 * @param tf
	 */
	public void setValues(int[] indexes, double[] tf) {
		if (indexes.length == tf.length) {
			this.indexes = indexes;
			this.tf = tf;
		}
	}

	/**
	 * Seta os valores do vetor, utilizando a frequencia dos termos encontrados.
	 * valor[x] = log(1 + frequency[x]);
	 * 
	 * @param indexes
	 * @param frequency
	 */
	public void setValues(int[] indexes, int[] frequency) {
		if (indexes.length == frequency.length) {
			double[] tf = new double[frequency.length];
			for (int i = 0; i < frequency.length; i++) {
				tf[i] = Math.log1p(frequency[i]);
			}
			this.setValues(indexes, tf);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public double[] getValues() {
		return this.tf;
	}

	/**
	 * {@inheritDoc}
	 */
	public int[] getIndexes() {
		return this.indexes;
	}

}
