package bnrobot.core.controller;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import bnrobot.core.ClassificationResult;
import bnrobot.core.Document;
import bnrobot.core.models.DocumentElement;
import bnrobot.core.models.Subject;
import bnrobot.core.parser.DocumentParser;
import bnrobot.tests.Cronometro;

public class CollectionActor {

	private final CollectionController controller;

	public CollectionActor(CollectionController controller) {
		this.controller = controller;
	}

	public DocumentElement processDoc(Document document) {
		DocumentElement element = this.controller.getCollection().getDocument(
				document.getId());

		if (element != null)
			return element;

		DocumentParser parser = new DocumentParser(document,
				this.controller.getCollection(), this.controller.getAnalyzer());

		element = new DocumentElement();
		element.setId(document.getId());
		element.setValues(parser.getIndexes(), parser.getFrequency());

		for (int termId : element.getIndexes()) {
			this.controller.getCollection().incrementTermFrequency(termId);
		}

		return element;
	}

	private List<DocumentElement> processDocs(List<Document> documents) {
		List<DocumentElement> elements = new LinkedList<DocumentElement>();
		for (Document document : documents) {
			DocumentElement element = this.processDoc(document);
			elements.add(element);
		}
		return elements;
	}

	public List<DocumentElement> store(List<Document> documents) {
		List<DocumentElement> elements = this.processDocs(documents);
		this.controller.getCollection().queueAdd(elements);
		return elements;
	}

	public DocumentElement store(Document document) {
		DocumentElement element = this.processDoc(document);
		this.controller.getCollection().queueAdd(element);
		return element;
	}

	public Collection<DocumentElement> queueProcess() {
		Collection<DocumentElement> elements = this.controller.getCollection()
				.queueValues();

		Cronometro.CLUSTERING.start();
		Collection<Subject> subjects = this.controller.getClustering()
				.insereDocs(elements);
		Cronometro.CLUSTERING.stop();

		Cronometro.CLASSIFICATION.start();
		for (Subject subject : subjects) {
			if (subject.getLabel() == null) {
				ClassificationResult result = this.controller
						.getClassificator().categoryOf(subject,
								this.controller.getCollection());
				subject.setLabel(result.getClassification());
				subject.setScore(result.getScore());
			}
		}
		Cronometro.CLASSIFICATION.stop();

		Cronometro.SAVING.start();
		this.controller.getCollection().saveDocumentElements(elements);
		this.controller.getCollection().saveSubjects(subjects);
		Cronometro.SAVING.stop();

		return elements;
	}

}
