package bnrobot.core.controller;

import java.io.IOException;
import java.util.Properties;

import jdbm.RecordManager;
import jdbm.RecordManagerFactory;
import jdbm.RecordManagerOptions;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.snowball.SnowballAnalyzer;
import org.apache.lucene.util.Version;

import bnrobot.classification.knn.NearestNeighbor;
import bnrobot.clustering.hierarchical.HierarchicalClustering;
import bnrobot.configuration.CollectionConfiguration;
import bnrobot.core.Classificator;
import bnrobot.core.Clustering;
import bnrobot.core.DocCollection;
import bnrobot.tests.Cronometro;

/**
 * Classe para gerar o controller. <br>
 * 
 * Essa classe será apagada futuramente, quando começar a utilizar o spring
 * framework
 * 
 * @author willian
 * 
 */
public class CollectionControllerFactory {

	public CollectionController getController(CollectionConfiguration config)
			throws IOException {
		Cronometro.RELOADING.start();
		CollectionController controller = new CollectionController();
		controller.setAnalyzer(this.getAnalizer(config));
		controller.setRecman(this.getRecordManager(config));
		controller.setClassificator(this.getClassificator());
		controller.setCollection(this.getCollection(controller));
		controller.setClustering(this.getClustering(controller, config));
		Cronometro.RELOADING.stop();

		return controller;
	}

	private Clustering getClustering(CollectionController controller,
			CollectionConfiguration config) {
		return new HierarchicalClustering(controller.getCollection(), config);
	}

	private DocCollection getCollection(CollectionController controller) {
		return new DocCollection(controller);
	}

	private Classificator getClassificator() {
		return new NearestNeighbor();
	}

	private RecordManager getRecordManager(CollectionConfiguration config)
			throws IOException {
		Properties options = new Properties();
		options.setProperty(RecordManagerOptions.CACHE_TYPE, "none");

		return RecordManagerFactory.createRecordManager(config.getDbPath(),
				options);
	}

	private Analyzer getAnalizer(CollectionConfiguration config) {
		return new SnowballAnalyzer(Version.LUCENE_30,
				config.getAnalyzerName(), config.stopWords());
	}

}
