package bnrobot.core.models;

import bnrobot.core.ArrayOperations;
import bnrobot.core.Persistible;
import bnrobot.core.VSElementImpl;

/**
 * 
 * @author Willian Gigliotti
 * 
 */
public class Subject extends VSElementImpl implements Persistible {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private Integer id;
	private Integer familyId;
	private Double score = Double.valueOf(0);
	private Integer label = null;

	public void setId(Integer id) {
		this.id = id;
	}

	public Integer getId() {
		return id;
	}

	public void setFamilyId(Integer familyId) {
		this.familyId = familyId;
	}

	public Integer getFamilyId() {
		return familyId;
	}

	public void setLabel(Integer classification) {
		this.label = classification;
	}

	public Integer getLabel() {
		return label;
	}

	public void addDocumentElement(DocumentElement element) {
		this.computeTf(element);
	}

	private void computeTf(DocumentElement element) {
		int[] indexElement = element.getIndexes();
		double[] tfElement = element.getValues();

		int size = (int) ArrayOperations.unionsSize(this.indexes, indexElement);
		int[] newIndex = new int[size];
		double[] newTf = new double[size];

		int pa = 0, pb = 0, pi = 0;

		while (pa < this.indexes.length || pb < indexElement.length) {
			if ((pa >= this.indexes.length)
					|| (pb < indexElement.length && this.indexes[pa] > indexElement[pb])) {
				newIndex[pi] = indexElement[pb];
				newTf[pi] = tfElement[pb];
				pi++;
				pb++;
			} else if ((pb >= indexElement.length)
					|| (pa < this.indexes.length && indexElement[pb] > this.indexes[pa])) {
				newIndex[pi] = this.indexes[pa];
				newTf[pi] = this.tf[pa];
				pi++;
				pa++;
			} else {
				newIndex[pi] = this.indexes[pa];
				newTf[pi] = Math.log1p(Math.exp(this.tf[pa])
						+ Math.exp(tfElement[pb]) - 1);
				pi++;
				pa++;
				pb++;
			}
		}

		this.tf = newTf;
		this.indexes = newIndex;

	}

	public void setScore(Double score) {
		this.score = score;
	}

	public Double getScore() {
		return score;
	}

}
