package bnrobot.core.provider;

import bnrobot.core.models.Term;

/**
 * Interface para Provider de Termos
 * 
 * @author willian
 * 
 */
public interface TermProvider {

	/**
	 * Dado a string de um termo, retorna o ID dele
	 * 
	 * @param term
	 *            termo
	 * @return id do termo
	 */
	Integer getId(String term);

	/**
	 * @return Quantidade de termos
	 */
	int size();

	/**
	 * Incrementa em 1 a contagem do termo dado
	 * 
	 * @param key
	 *            id do termo
	 */
	void increment(Integer key);

	/**
	 * Retorna um vetor DF de frequencia de termos, onde DF[t] é a quantidade de
	 * documentos em que o termo com id 't' é encontrado, de acordo com o método
	 * {@link TermProvider#increment(Integer)}
	 * 
	 * @return vetor de frequencia dos termos
	 */
	int[] frequencies();

	/**
	 * Commit
	 */
	void commit();

	/**
	 * Retorna o vetor T[i], com os termos, onde T[i] é o Term de id = i
	 * 
	 * @return vetor de termos
	 */
	Term[] terms();

}
