package bnrobot.jdbm.persister;

import jdbm.PrimaryTreeMap;
import jdbm.RecordManager;
import jdbm.SecondaryHashMap;
import jdbm.SecondaryKeyExtractor;
import bnrobot.core.models.DocumentElement;
import bnrobot.core.provider.DocumentProvider;

/**
 * 
 * @author Willian Gigliotti
 * 
 */
public class DocumentElementDao implements DocumentProvider {

	PrimaryTreeMap<Integer, DocumentElement> documents;
	SecondaryHashMap<Integer, Integer, DocumentElement> documentsBySubject;

	public DocumentElementDao(RecordManager recman) {
		// documents = recman.storeMap("documents", this);
		this.documents = recman.treeMap("documents");

		this.documentsBySubject = this.documents.secondaryHashMap(
				"documentsBySubject",
				new SecondaryKeyExtractor<Integer, Integer, DocumentElement>() {

					public Integer extractSecondaryKey(Integer key,
							DocumentElement doc) {
						return doc.getSubjectId();
					}
				});

	}

	public int save(DocumentElement de) {
		this.documents.put(de.getId(), de);
		return de.getId();
	}

	public DocumentElement getDocumentElement(int id) {
		return this.documents.get(id);
	}

	public int size() {
		return this.documents.size();
	}

	public Iterable<DocumentElement> documents() {
		return this.documents.values();
	}

	public Iterable<DocumentElement> findBySubject(Integer subject) {
		return this.documentsBySubject.getPrimaryValues(subject);

	}
}
