package bnrobot.jdbm.persister;

import java.io.IOException;
import java.util.Collection;
import java.util.Map;

import jdbm.PrimaryTreeMap;
import jdbm.RecordManager;
import jdbm.SecondaryHashMap;
import jdbm.SecondaryKeyExtractor;
import jdbm.Serializer;
import jdbm.SerializerInput;
import jdbm.SerializerOutput;
import bnrobot.core.models.Family;
import bnrobot.core.models.Subject;

/**
 * 
 * @author Willian Gigliotti
 * 
 */
public class SubjectDao implements Serializer<Subject> {

	PrimaryTreeMap<Integer, Subject> subjects;
	SecondaryHashMap<Integer, Integer, Subject> subjectsByFamily;

	public SubjectDao(RecordManager recman) {
		subjects = recman.treeMap("subjects");

		subjectsByFamily = subjects.secondaryHashMap("subjectsByFamily",
				new SecondaryKeyExtractor<Integer, Integer, Subject>() {

					public Integer extractSecondaryKey(Integer key, Subject subject) {
						return subject.getFamilyId();
					}
				});
	}

	public Subject deserialize(SerializerInput in) throws IOException,
			ClassNotFoundException {
		return in.readObject();
	}

	public void serialize(SerializerOutput out, Subject subject)
			throws IOException {
		out.writeObject(subject);
	}

	public Subject subjectById(Integer id) {
		return subjects.get(id);
	}

	public Iterable<Subject> subjectsByFamily(Family family) {
		return subjectsByFamily(family.getId());
	}

	public Iterable<Subject> subjectsByFamily(int family) {
		return subjectsByFamily.getPrimaryValues(family);
	}

	public int size() {
		return subjects.size();
	}

	public int nextKey() {
		return subjects.size() == 0 ? 1 : subjects.lastKey();
	}

	public void putAll(Map<Integer, Subject> map) {
		subjects.putAll(map);
	}

	public void remove(Integer key) {
		subjects.remove(key);
	}

	public Collection<Subject> getSubjects() {
		return subjects.values();
	}

}
