package bnrobot.jdbm.persister;

import jdbm.PrimaryHashMap;
import jdbm.RecordManager;
import jdbm.SecondaryHashMap;
import jdbm.SecondaryKeyExtractor;
import bnrobot.core.models.Term;

/**
 * A maneira que o TermDao lida com o Model é diferente dos outros, ela faz uso
 * do Model Dao, porém não oferece uma interface externa de acesso para esses
 * objetos, no lugar fornece algumas operações, a fim de garantir a consistência
 * do conjunto.
 * 
 * @author Willian Gigliotti
 * 
 */
public class TermDao {
	PrimaryHashMap<Integer, Term> terms;
	SecondaryHashMap<String, Integer, Term> termByString;

	public TermDao(RecordManager recman) {
		// terms = recman.storeMap("terms", this);
		terms = recman.hashMap("terms");
		termByString = terms.secondaryHashMap("termByString",
				new SecondaryKeyExtractor<String, Integer, Term>() {

					public String extractSecondaryKey(Integer key, Term term) {
						return term.getString();
					}
				});
	}

	public int save(Term term) {
		if (term.getId() == null) {
			int key = terms.size() + 1;
			term.setId(key);
		}
		terms.put(term.getId(), term);
		return term.getId();
	}

	public Integer idByString(String term) {
		if (termByString.containsKey(term)) {
			return termByString.get(term).iterator().next();
		}
		Term novo = new Term();
		novo.setString(term);
		return this.save(novo);
	}

	public void increment(Integer key) {
		if (terms.containsKey(key)) {
			Term t = terms.get(key);
			t.increment();
			this.save(t);
		}
	}

	public Integer termFrequency(Integer key) {
		if (terms.containsKey(key)) {
			Term t = terms.get(key);
			return t.getCount();
		}
		return 0;
	}

	public int size() {
		return terms.size();
	}

	public Iterable<Term> elements() {
		return terms.values();
	}

}
