package bnrobot.jdbm.provider;

import java.util.HashMap;
import java.util.Map;

import bnrobot.core.models.Family;
import bnrobot.core.provider.FamilyProvider;
import bnrobot.jdbm.persister.FamilyDao;

public class FamilyCache implements FamilyProvider {
	private final FamilyDao dao;
	private final Map<Integer, Family> families;

	private int nextKey;

	public FamilyCache(FamilyDao dao) {
		this.dao = dao;
		this.families = new HashMap<Integer, Family>(dao.map());
		this.nextKey = dao.nextKey();
	}

	public Iterable<Family> families() {
		return this.families.values();
	}

	public int size() {
		return this.families.size();
	}

	public int save(Family f) {
		Integer key = f.getId();
		if (key == null) {
			key = this.nextKey++;
			f.setId(key);
		}
		this.families.put(key, f);
		return key;
	}

	public void commit() {
		this.dao.putAll(this.families);
	}

}
