package bnrobot.jdbm.provider;

import java.util.HashMap;
import java.util.Map;

import bnrobot.core.models.Subject;
import bnrobot.core.provider.SubjectProvider;
import bnrobot.jdbm.persister.SubjectDao;

public class SubjectCache implements SubjectProvider {

	/**
	 * Próxima chave para os objetos
	 */
	private int nextKey;

	/**
	 * Objecto DAO de Subjects, usado para fazer a persistência
	 */
	private final SubjectDao dao;

	private Map<Integer, Subject> history = new HashMap<Integer, Subject>();

	public SubjectCache(SubjectDao dao) {
		this.dao = dao;
		this.nextKey = dao.nextKey();
	}

	public int size() {
		// arrumar isso
		return this.nextKey - 1;
	}

	public Integer save(Subject subject) {
		Integer key = subject.getId();
		if (key == null) {
			key = this.nextKey++;
			subject.setId(key);
		}
		this.history.put(key, subject);
		return key;
	}

	public void commit() {
		this.dao.putAll(this.history);
		this.history = new HashMap<Integer, Subject>();
	}

	public Subject getSubject(int id) {
		Subject sj = this.history.get(id);
		if (sj == null) {
			sj = this.dao.subjectById(id);
		}
		return sj;

	}

	public Iterable<Subject> subjects() {
		return this.dao.getSubjects();
	}

	public void remove(int id) {
		if (this.history.containsKey(id)) {
			this.history.remove(id);
		}
		this.dao.remove(id);
	}

}
