package bnrobot.jdbm.provider;

import java.util.HashMap;

import bnrobot.core.models.Term;
import bnrobot.core.provider.TermProvider;
import bnrobot.jdbm.persister.TermDao;

public class TermCache implements TermProvider {
	private final TermDao dao;

	private final HashMap<String, Integer> ids = new HashMap<String, Integer>();
	private Term[] cache;
	private int[] frequency;

	public TermCache(TermDao dao) {
		this.dao = dao;

		this.cache = new Term[dao.size() * 2 + 100];
		this.frequency = new int[dao.size() * 2 + 100];

		for (Term term : dao.elements()) {
			this.cache[term.getId()] = term;
			this.ids.put(term.getString(), term.getId());
		}
		System.out.println("Existem: " + dao.size() + " termos salvos");
	}

	private void doubleCapacity() {
		int size = this.frequency.length;
		int[] freq = new int[2 * size];
		Term[] terms = new Term[2 * size];

		for (int i = 0; i < size; i++) {
			freq[i] = this.frequency[i];
			terms[i] = this.cache[i];
		}
		this.frequency = freq;
		this.cache = terms;
	}

	public Integer getId(String term) {
		Integer key = this.ids.get(term);
		if (key == null) {
			key = this.ids.size() + 1;
			while (key >= this.frequency.length) {
				this.doubleCapacity();
			}
			Term val = new Term();
			val.setId(key);
			val.setString(term);
			this.cache[key] = val;

			this.ids.put(term, key);
		}
		return key;
	}

	public int size() {
		return this.ids.size();
	}

	public void increment(Integer key) {
		this.frequency[key]++;
		this.cache[key].increment();
	}

	public int[] frequencies() {
		return this.frequency;
	}

	public void commit() {
		for (int i = 1; i <= this.ids.size(); i++) {
			this.dao.save(this.cache[i]);
		}
		System.out.println("Salvo " + this.ids.size() + " termos");
	}

	public Term[] terms() {
		return this.cache;
	}

}
