package bnrobot.mocks;

import java.util.Map.Entry;
import java.util.TreeMap;

import bnrobot.core.TfIdfElement;
import bnrobot.vectorSpace.VSElement;

/**
 * Implementação simples de um {@link VSElement}
 * 
 * @author Willian Gigliotti
 * 
 */
public class VSMock extends TfIdfElement<VSElement> {

	public VSMock() {
		super(null, null);
	}

	private int[] indexCache = null;
	private double[] valuesCache = null;

	private final TreeMap<Integer, Double> values = new TreeMap<Integer, Double>();

	private void computeValues() {
		if (this.indexCache != null) {
			return;
		}

		this.indexCache = new int[this.values.size()];
		this.valuesCache = new double[this.values.size()];

		int i = 0;
		double sum = 0;
		for (Entry<Integer, Double> entry : this.values.entrySet()) {
			this.indexCache[i] = entry.getKey();
			this.valuesCache[i] = entry.getValue();
			sum += entry.getValue() * entry.getValue();
			i++;
		}

		if (sum > 0) {
			double size = Math.sqrt(sum);
			for (i = 0; i < this.valuesCache.length; i++) {
				this.valuesCache[i] /= size;
			}
		}
	}

	@Override
	public int[] getIndexes() {
		this.computeValues();
		return this.indexCache;
	}

	@Override
	public double[] getValues() {
		this.computeValues();
		return this.valuesCache;
	}

	public VSMock setValue(Integer index, Double value) {
		this.values.put(index, value);
		this.indexCache = null;
		return this;
	}

	/**
	 * @return Norma euclidiana do vetor
	 */
	public double getLength() {
		this.computeValues();
		double size = 0;
		for (int i = 0; i < this.valuesCache.length; i++) {
			size += this.valuesCache[i] * this.valuesCache[i];
		}
		return Math.sqrt(size);
	}

	/**
	 * @return um Vetor zerado, para testes
	 */
	public static VSMock zero() {
		VSMock mock = new VSMock();
		mock.setValue(0, 0D);
		return mock;
	}

}
