package br.com.w2it.filter;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import bnrobot.configuration.CollectionConfiguration;
import bnrobot.core.controller.CollectionController;
import bnrobot.core.controller.CollectionControllerFactory;
import bnrobot.tests.Cronometro;

/**
 * Essa é a classe que deve ser executada para rodar os testes. <br>
 * 
 * Para mudar alguma configuração deve ser alterada a classe Configuration.
 */
public class App {
	private final Connection con;
	private final CollectionController controller;
	private final Configuration config;

	private final RobotActor actor;

	public App() throws ClassNotFoundException, SQLException, IOException {
		this.config = new Configuration();
		this.con = this.getConnection();
		this.controller = new CollectionControllerFactory()
				.getController(new CollectionConfiguration());

		this.actor = new RobotActor(this.con, this.controller);
	}

	public void close() throws SQLException, IOException {
		this.con.close();
		this.controller.getCollection().close();
	}

	private Connection getConnection() throws ClassNotFoundException,
			SQLException {
		Class.forName("com.mysql.jdbc.Driver");

		String url = this.config.getUrl();

		Connection con = DriverManager.getConnection(url,
				this.config.getUsername(), this.config.getPassword());
		return con;
	}

	private void train() throws SQLException {
		this.actor.train();
	}

	private void clusterize() throws SQLException, IOException {
		this.actor.processAndArchive(this.config.getMaxStep());
	}

	public void deleteOld() throws SQLException {
		this.actor.deleteOld();
	}

	public static void main(String[] args) throws ClassNotFoundException,
			SQLException, IOException {

		Cronometro.TOTAL.start();
		App clustering = new App();

		clustering.train();
		clustering.clusterize();

		// TODO descomentar a linha de baixo quando ficar pronto
		// Ela que é responsável para apagar as notícias velhas...
		// clustering.deleteOld();

		Cronometro.CLOSE.start();
		clustering.close();
		Cronometro.CLOSE.stop();

		Cronometro.TOTAL.stop();
		Cronometro.showTimes();
	}
}
