package br.com.w2it.filter;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.TreeSet;

import bnrobot.configuration.CollectionConfiguration;
import bnrobot.core.DocCollection;
import bnrobot.core.Similarity;
import bnrobot.core.TfIdfElement;
import bnrobot.core.controller.CollectionController;
import bnrobot.core.controller.CollectionControllerFactory;
import bnrobot.core.models.DocumentElement;
import bnrobot.core.models.Subject;
import bnrobot.tests.Cronometro;

public class DocSimApp {

	private final Connection con;
	private final CollectionController controller;
	private final Configuration config;

	public DocSimApp() throws ClassNotFoundException, SQLException, IOException {
		this.config = new Configuration();
		this.con = this.getConnection();
		this.controller = new CollectionControllerFactory()
				.getController(new CollectionConfiguration());

	}

	private Connection getConnection() throws ClassNotFoundException,
			SQLException {
		Class.forName("com.mysql.jdbc.Driver");

		String url = this.config.getUrl();

		Connection con = DriverManager.getConnection(url,
				this.config.getUsername(), this.config.getPassword());
		return con;
	}

	public static void main(String[] args) throws ClassNotFoundException,
			SQLException, IOException {

		Cronometro.TOTAL.start();
		DocSimApp sim = new DocSimApp();

		// clustering.train();
		sim.compute();

		// TODO descomentar a linha de baixo quando ficar pronto
		// Ela que é responsável para apagar as notícias velhas...
		// clustering.deleteOld();

		Cronometro.CLOSE.start();
		Cronometro.CLOSE.stop();

		Cronometro.TOTAL.stop();
		Cronometro.showTimes();
	}

	private void compute() {
		int tamanho = 10;
		int[] distribuicaoIn = new int[tamanho];
		int[] distribuicaoOut = new int[tamanho];

		DocCollection collection = this.controller.getCollection();
		TreeSet<Similarity<Subject, DocumentElement>> similarities = new TreeSet<Similarity<Subject, DocumentElement>>();

		Subject sub = collection.getSubject(2311);
		TfIdfElement<Subject> tf = new TfIdfElement<Subject>(
				collection.getIdf(), sub);

		System.out.println("DOCS: " + collection.getDocumentProvider().size());

		int j = 0;
		for (DocumentElement doc : collection.getDocumentProvider().documents()) {

			Similarity<Subject, DocumentElement> sim = new Similarity<Subject, DocumentElement>(
					tf, doc, collection);
			int disPos = this.getDistriPosition(tamanho, sim.getValue());
			boolean in = sub.getId().equals(doc.getSubjectId());
			if (in) {
				distribuicaoIn[disPos]++;
			} else {
				distribuicaoOut[disPos]++;
				if (disPos >= 3) {
					similarities.add(sim);
				}
			}

		}
		System.out.println(similarities.size());
		for (Similarity<Subject, DocumentElement> similarity : similarities
				.descendingSet()) {
			System.out.print(similarity.getElementB().getId() + ", ");
		}
		System.out.println("\n");

		for (int i = 0; i < tamanho; i++) {
			System.out.println(i + " :" + distribuicaoOut[i] + "  |  "
					+ distribuicaoIn[i]);
		}

	}

	private int getDistriPosition(int tamanho, double value) {
		double intervalo = 1.0 / tamanho;
		int val = (int) (value / intervalo);
		return val > (tamanho - 1) ? (tamanho - 1) : val;

	}

}
