package br.com.w2it.filter;

import java.io.IOException;
import java.sql.SQLException;

import bnrobot.core.DocCollection;
import bnrobot.core.TfIdfElement;
import bnrobot.core.models.Subject;
import bnrobot.vectorSpace.VSElement;

public class Projecao {

	public static void main(String[] args) throws ClassNotFoundException,
			SQLException, IOException {
		AppStarter app = new AppStarter();
		Integer doc1 = 2243;
		Integer doc2 = 2265;
		Integer doc3 = 2174;
		getPoints(app, doc1, doc2, doc3);
	}

	private static void getPoints(AppStarter app, final Integer doc1,
			final Integer doc2, final Integer doc3) {
		DocCollection collection = app.getController().getCollection();
		double[] idf = collection.getIdf();

		TfIdfElement<Subject> sub1 = new TfIdfElement<Subject>(idf,
				collection.getSubject(doc1));
		TfIdfElement<Subject> sub2 = new TfIdfElement<Subject>(idf,
				collection.getSubject(doc2));
		TfIdfElement<Subject> sub3 = new TfIdfElement<Subject>(idf,
				collection.getSubject(doc3));

		double[] e1 = normaliza(menos(sub2, sub1, idf.length));
		double[] e2 = normaliza(ortogonal(menos(sub3, sub1, idf.length), e1));

		// projetar(collection.getDocumentDao().findAll(), idf, e1, e2, sub1,
		// new Condicao<DocumentElement>() {
		//
		// public boolean can(DocumentElement object) {
		// if (object.getSubjectId().equals(doc1)
		// || object.getSubjectId().equals(doc2)
		// || object.getSubjectId().equals(doc3)) {
		// return false;
		// }
		// return true;
		// }
		// });

		// projetar(app, doc1, e1, e2, sub1);
		// projetar(app, doc2, e1, e2, sub1);
		// projetar(app, doc3, e1, e2, sub1);

		projetar(collection.getActiveSubjects(), idf, e1, e2, sub1);

	}

	private static void projetar(AppStarter app, Integer doc1, double[] e1,
			double[] e2, TfIdfElement<Subject> sub1) {
		double[] idf = app.getController().getCollection().getIdf();
		System.out.println("\n\n" + doc1 + "\n\n");
		projetar(app.getController().getCollection().getDocumentProvider()
				.findBySubject(doc1), idf, e1, e2, sub1);
	}

	private static <VS extends VSElement> void projetar(Iterable<VS> elementos,
			double[] idf, double[] e1, double[] e2,
			TfIdfElement<? extends VSElement> base) {
		projetar(elementos, idf, e1, e2, base, new Condicao<VS>() {

			public boolean can(VS object) {
				return true;
			}
		});
	}

	private static <VS extends VSElement> void projetar(Iterable<VS> elementos,
			double[] idf, double[] e1, double[] e2,
			TfIdfElement<? extends VSElement> base, Condicao<VS> condicao) {
		for (VS element : elementos) {
			if (condicao.can(element)) {
				projetar(element, idf, e1, e2, base);
			}
		}
	}

	private static void projetar(VSElement element, double[] idf, double[] e1,
			double[] e2, TfIdfElement<? extends VSElement> base) {
		TfIdfElement<VSElement> elemento = new TfIdfElement<VSElement>(idf,
				element);
		double[] tmp = menos(elemento, base, idf.length);

		double a1 = produtoEscalar(tmp, e1);
		double a2 = produtoEscalar(tmp, e2);

		System.out.println(a1 + ", " + a2);
	}

	private static double[] ortogonal(double[] e2, double[] e1) {
		double alpha = produtoEscalar(e2, e1);
		for (int i = 0; i < e2.length; i++) {
			e2[i] = e2[i] - alpha * e1[i];
		}
		return e2;
	}

	private static double produtoEscalar(double[] e2, double[] e1) {
		double prod = 0;
		for (int i = 0; i < e1.length; i++) {
			prod += e1[i] * e2[i];
		}
		return prod;
	}

	private static double[] menos(TfIdfElement<? extends VSElement> sub2,
			TfIdfElement<? extends VSElement> sub1, int length) {
		double[] diferenca = new double[length];
		return menos(sub2, sub1, diferenca);
	}

	private static double[] menos(TfIdfElement<? extends VSElement> sub2,
			TfIdfElement<? extends VSElement> sub1, double[] diferenca) {
		int[] index = sub2.getIndexes();
		double[] value = sub2.getValues();

		for (int i = 0; i < index.length; i++) {
			diferenca[index[i]] += value[i];
		}

		index = sub1.getIndexes();
		value = sub1.getValues();

		for (int i = 0; i < index.length; i++) {
			diferenca[index[i]] -= value[i];
		}
		return diferenca;
	}

	private static double[] normaliza(double[] vetor) {
		double norma = Math.sqrt(produtoEscalar(vetor, vetor));
		for (int i = 0; i < vetor.length; i++) {
			vetor[i] = vetor[i] / norma;
		}
		return vetor;
	}

	interface Condicao<E> {
		boolean can(E object);
	}

}
