package br.com.w2it.filter;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;

import bnrobot.core.Document;
import bnrobot.core.controller.CollectionController;
import bnrobot.core.models.DocumentElement;
import bnrobot.core.models.Subject;
import bnrobot.tests.Cronometro;

public class RobotActor {

	private final Connection connection;
	private final CollectionController controller;
	private final RobotQueries queries;

	public RobotActor(Connection connection, CollectionController controller) {
		this.connection = connection;
		this.controller = controller;
		this.queries = new RobotQueries();
	}

	private void carregaFila(int maxStep) throws SQLException {
		Statement stmt = this.connection.createStatement();

		String query = this.queries.findDocumentQueue(maxStep);
		ResultSet result = stmt.executeQuery(query);

		while (result.next()) {
			String string = result.getString("bn_content");
			int id = result.getInt("bn_id");

			Document t = new Document(id, string);
			this.controller.queueAdd(t);
		}

		stmt.close();
	}

	/**
	 * Método que carrega a fila do robo e faz o processamento dela.
	 * 
	 * @param maxStep
	 *            quantidade máxima de documentos que serão processados
	 * @throws SQLException
	 * @throws IOException
	 */
	public void processAndArchive(int maxStep) throws SQLException, IOException {
		Cronometro.LOADING.start();
		this.carregaFila(maxStep);
		Cronometro.LOADING.stop();

		Collection<DocumentElement> elements = this.controller.processarFila();

		this.queueToArchive(elements);
		this.controller.getCollection().commit();
	}

	/**
	 * Passa os elementos da Fila de processamento para a tabela de arquivamento
	 * 
	 * @param elements
	 * @throws SQLException
	 */
	private void queueToArchive(Collection<DocumentElement> elements)
			throws SQLException {
		Statement stmt = this.connection.createStatement();

		for (DocumentElement element : elements) {
			this.queueToArchive(stmt, element);
		}

		String query = this.queries.updateSize();
		stmt.executeUpdate(query);

		stmt.close();
	}

	/**
	 * Move um elemento da fila de processamento para o arquivo
	 * 
	 * @param stmt
	 * @param element
	 * @throws SQLException
	 */
	private void queueToArchive(Statement stmt, DocumentElement element)
			throws SQLException {
		String query = this.queries.findDocumentById(element.getId());
		ResultSet result = stmt.executeQuery(query);
		if (!result.next()) {
			throw new SQLException("Elemento não encontrado. ID:"
					+ element.getId());
		}

		String data = result.getString(1);

		this.updateSubject(stmt, element, data);

		query = this.queries.updateSubjectFromQueue(element.getId(),
				element.getSubjectId());
		stmt.executeUpdate(query);

		try {
			query = this.queries.insertArchiveFromQueue(element.getId());
			stmt.executeUpdate(query);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		query = this.queries.deleteFromQueue(element.getId());
		stmt.executeUpdate(query);
	}

	/**
	 * Faz atualizações do Subject ou cria um novo quando necessário
	 * 
	 * @param stmt
	 * @param element
	 * @throws SQLException
	 */
	private void updateSubject(Statement stmt, DocumentElement element,
			String data) throws SQLException {

		String query = this.queries.findSubjectById(element.getSubjectId());
		ResultSet result = stmt.executeQuery(query);

		if (result.next()) {
			query = this.queries.updateSubject(element.getId(),
					element.getSubjectId(), data);
		} else {
			Subject subject = this.controller.getCollection().getSubject(
					element.getSubjectId());

			query = this.queries.insertSubject(subject.getId(),
					element.getId(), subject.getLabel(), subject.getScore(),
					data);
		}

		stmt.executeUpdate(query);
	}

	/**
	 * Executa o processo de treinamento
	 * 
	 * @throws SQLException
	 */
	public void train() throws SQLException {

		System.out.println("Carregando treino " + Cronometro.LOADING.getTime());
		Statement stmt = this.connection.createStatement();
		Statement train = this.connection.createStatement();
		while (true) {

			Cronometro.LOADING.start();
			String query = this.queries.findTrainingQueue(5000);
			ResultSet result = stmt.executeQuery(query);
			Cronometro.LOADING.stop("rodando query");

			Integer id = null;

			Cronometro.LOADING.start();
			while (result.next()) {
				String string = result.getString("bn_content");
				id = result.getInt("bn_id");
				Integer label = result.getInt("bn_label");

				Document t = new Document(id, string);
				this.controller.getCollection().setCategory(t, label);

			}

			Cronometro.LOADING.stop("Cadastrando " + id);

			if (id != null) {
				query = this.queries.insertTrainingfromQueue(id);
				train.executeUpdate(query);

				query = this.queries.deleteFromTraningQueue(id);
				train.executeUpdate(query);
			} else {
				break;
			}

		}

		try {
			this.controller.recmanCommit();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		train.close();
		stmt.close();
	}

	/**
	 * Remove os subjects antigos
	 * 
	 * @throws SQLException
	 */
	public void deleteOld() throws SQLException {
		String query = "SELECT sub.bn_id as bn_id FROM `Subject` sub, (select date_sub(max(bn_last_date), interval 2 day) as max from Subject ss) max WHERE sub.bn_last_date < max.max and sub.bn_old <> 1";
		Statement stmt = this.connection.createStatement();
		ResultSet result = stmt.executeQuery(query);

		Statement old = this.connection.createStatement();
		int count = 0;
		while (result.next()) {
			int id = result.getInt("bn_id");
			this.controller.getCollection().removeSubject(id);

			query = "update Subject set bn_old = 1 where bn_id = " + id;
			old.executeUpdate(query);
			count++;
		}
		System.out.println("Grupos apagados: " + count);

	}

}
